MODULE usrdef_zgr
   !!======================================================================
   !!                       ***  MODULE  usrdef_zgr  ***
   !!
   !!                       ===  AM98 configuration  ===
   !!
   !! User defined : vertical coordinate system of a user configuration
   !!======================================================================
   !! History :  4.0  ! 2016-06  (G. Madec)  Original code
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   usr_def_zgr   : user defined vertical coordinate system
   !!      zgr_z      : reference 1D z-coordinate
   !!      zgr_top_bot: ocean top and bottom level indices
   !!      zgr_zco    : 3D verticl coordinate in pure z-coordinate case
   !!---------------------------------------------------------------------
   USE oce            ! ocean variables
   USE dom_oce        ! ocean domain
   USE depth_e3       ! depth <=> e3
   USE usrdef_nam
   !
   USE in_out_manager ! I/O manager
   USE lbclnk         ! ocean lateral boundary conditions (or mpp link)
   USE lib_mpp        ! distributed memory computing library

   USE usrdef_nam , ONLY : rn_dx, nn_AM98, rn_abp, r1_abp    ! use penalisation parameter

   IMPLICIT NONE
   PRIVATE
   !
   INTEGER, PARAMETER, PRIVATE  ::   nT  = 1
   INTEGER, PARAMETER, PRIVATE  ::   nU  = 2
   INTEGER, PARAMETER, PRIVATE  ::   nV  = 3
   INTEGER, PARAMETER, PRIVATE  ::   nF  = 4
   !
   PUBLIC   usr_def_zgr        ! called by domzgr.F90

   !!----------------------------------------------------------------------
   !! NEMO/OCE 4.0 , NEMO Consortium (2018)
   !! $Id: usrdef_zgr.F90 10425 2018-12-19 21:54:16Z smasson $
   !! Software governed by the CeCILL license (see ./LICENSE)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE usr_def_zgr( ld_zco  , ld_zps  , ld_sco  , ld_isfcav,    &   ! type of vertical coordinate
      &                    pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d  ,    &   ! 1D reference vertical coordinate
      &                    pdept , pdepw ,                             &   ! 3D t & w-points depth
      &                    pe3t  , pe3u  , pe3v   , pe3f ,             &   ! vertical scale factors
      &                    pe3w  , pe3uw , pe3vw         ,             &   !     -      -      -
      &                    k_top , k_bot                 )                 ! top & bottom ocean level
      !!---------------------------------------------------------------------
      !!              ***  ROUTINE usr_def_zgr  ***
      !!
      !! ** Purpose :   User defined the vertical coordinates
      !!
      !!----------------------------------------------------------------------
      LOGICAL                   , INTENT(out) ::   ld_zco, ld_zps, ld_sco      ! vertical coordinate flags
      LOGICAL                   , INTENT(out) ::   ld_isfcav                   ! under iceshelf cavity flag
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pdept_1d, pdepw_1d          ! 1D grid-point depth     [m]
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pe3t_1d , pe3w_1d           ! 1D grid-point depth     [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pdept, pdepw                ! grid-point depth        [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pe3t , pe3u , pe3v , pe3f   ! vertical scale factors  [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pe3w , pe3uw, pe3vw         ! i-scale factors
      INTEGER , DIMENSION(:,:)  , INTENT(out) ::   k_top, k_bot                ! first & last ocean level
      !
      INTEGER  ::   inum   ! local logical unit
      REAL(WP) ::   z_zco, z_zps, z_sco, z_cav
      REAL(wp), DIMENSION(jpi,jpj) ::   z2d   ! 2D workspace
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'usr_def_zgr : AM98 configuration (z-coordinate closed flat box ocean without cavities)'
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~'
      !
      !
      ! type of vertical coordinate
      ! ---------------------------
      ld_zco    = .FALSE.         ! AM98 case:  z-coordinate without ocean cavities
      ld_zps    = .FALSE.
      ld_sco    = .TRUE.
      ld_isfcav = .FALSE.
      !
      !
      ! Build the vertical coordinate system
      ! ------------------------------------
      CALL zgr_z( pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d )   ! Reference z-coordinate system
      !
      CALL zgr_msk_top_bot( k_top , k_bot)                 ! masked top and bottom ocean t-level indices
      !
      !                                                     ! z-coordinate (3D arrays) from the 1D z-coord.
      CALL zgr_zco( pdept_1d, pdepw_1d, pe3t_1d, pe3w_1d,   &   ! in  : 1D reference vertical coordinate
         &          pdept   , pdepw   ,                     &   ! out : 3D t & w-points depth
         &          pe3t    , pe3u    , pe3v   , pe3f   ,   &   !       vertical scale factors
         &          pe3w    , pe3uw   , pe3vw             )     !           -      -      -
      !
   END SUBROUTINE usr_def_zgr


   SUBROUTINE zgr_z( pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d )   ! 1D reference vertical coordinate
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE zgr_z  ***
      !!
      !! ** Purpose :   set the 1D depth of model levels and the resulting
      !!              vertical scale factors.
      !!
      !! ** Method  :   1D z-coordinate system (use in all type of coordinate)
      !!       The depth of model levels is set from dep(k), an analytical function:
      !!                   w-level: depw_1d  = dep(k)
      !!                   t-level: dept_1d  = dep(k+0.5)
      !!       The scale factors are the discrete derivative of the depth:
      !!                   e3w_1d(jk) = dk[ dept_1d ]
      !!                   e3t_1d(jk) = dk[ depw_1d ]
      !!           with at top and bottom :
      !!                   e3w_1d( 1 ) = 2 * ( dept_1d( 1 ) - depw_1d( 1 ) )
      !!                   e3t_1d(jpk) = 2 * ( dept_1d(jpk) - depw_1d(jpk) )
      !!       The depth are then re-computed from the sum of e3. This ensures
      !!    that depths are identical when reading domain configuration file.
      !!    Indeed, only e3. are saved in this file, depth are compute by a call
      !!    to the e3_to_depth subroutine.
      !!
      !!       Here the Madec & Imbard (1996) function is used.
      !!
      !! ** Action  : - pdept_1d, pdepw_1d : depth of T- and W-point (m)
      !!              - pe3t_1d , pe3w_1d  : scale factors at T- and W-levels (m)
      !!
      !! Reference : Marti, Madec & Delecluse, 1992, JGR, 97, No8, 12,763-12,766.
      !!             Madec and Imbard, 1996, Clim. Dyn.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pdept_1d, pdepw_1d   ! 1D grid-point depth        [m]
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pe3t_1d , pe3w_1d    ! 1D vertical scale factors  [m]
      !
      INTEGER  ::   jk       ! dummy loop indices
        !!----------------------------------------------------------------------
      !
        !
      IF(lwp) THEN            ! Parameter print
         WRITE(numout,*)
         WRITE(numout,*) '    zgr_z   : Reference vertical z-coordinates '
         WRITE(numout,*) '    ~~~~~~~'
      ENDIF

      !
      ! 1D Reference z-coordinate    (using Madec & Imbard 1996 function)
      ! -------------------------
      !
      ! depth at T and W-points
      pdepw_1d(1) =   0._wp
      pdept_1d(1) = 250._wp
      !
      pdepw_1d(2) = 500._wp
      pdept_1d(2) = 750._wp
      !
      !                       ! e3t and e3w from depth
      CALL depth_to_e3( pdept_1d, pdepw_1d, pe3t_1d, pe3w_1d )
      !
      !                       ! recompute depths from SUM(e3)  <== needed
      CALL e3_to_depth( pe3t_1d, pe3w_1d, pdept_1d, pdepw_1d )
      !
      IF(lwp) THEN                        ! control print
         WRITE(numout,*)
         WRITE(numout,*) '              Reference 1D z-coordinate depth and scale factors:'
         WRITE(numout, "(9x,' level  gdept_1d  gdepw_1d  e3t_1d   e3w_1d  ')" )
         WRITE(numout, "(10x, i4, 4f9.2)" ) ( jk, pdept_1d(jk), pdepw_1d(jk), pe3t_1d(jk), pe3w_1d(jk), jk = 1, jpk )
      ENDIF
      !
   END SUBROUTINE zgr_z




   SUBROUTINE zgr_msk_top_bot( k_top , k_bot)
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE zgr_msk_top_bot  ***
      !!
      !! ** Purpose :   set the masked top and bottom ocean t-levels
      !!
      !! ** Method  :   AM98 case = closed flat box ocean without ocean cavities
      !!                   k_top = 1     except along north, south, east and west boundaries
      !!                   k_bot = jpk-1 except along north, south, east and west boundaries
      !!
      !! ** Action  : - k_top : first wet ocean level index
      !!              - k_bot : last  wet ocean level index
      !!----------------------------------------------------------------------
      INTEGER , DIMENSION(:,:), INTENT(out) ::   k_top , k_bot   ! first & last wet ocean level
      !
      REAL(wp), DIMENSION(jpi,jpj) ::   z2d   ! 2D local workspace
      INTEGER  ::   ji, jj,jk, jl                    ! dummy loop indices
      REAL(wp) ::   zylim0, zylim1, zxlim0, zxlim1 ! limit of the domain [m]
      REAL(WP) ::   zcoeff    ! local scalar
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '    zgr_top_bot : defines the top and bottom wet ocean levels.'
      IF(lwp) WRITE(numout,*) '    ~~~~~~~~~~~'
      IF(lwp) WRITE(numout,*) '       AM98 case : closed flat box ocean without ocean cavities'
      !
      z2d(:,:) = REAL( jpkm1 , wp )          ! flat bottom
      !
      CALL lbc_lnk( 'usrdef_zgr', z2d, 'T', 1. )           ! set surrounding land to zero (here jperio=0 ==>> closed)
      !
      !
# if defined key_bvp
      ! Faire la pénalisation
      ! ktop = where phi flotte < 1% puis envoie au mask
      !  Penalize the first inner layer of fluid
      ! -----------------------------------------------------
      zylim0 =  10000._wp     ! +   10km
      zylim1 = 2010000._wp    !   2010km
      zxlim0 = -200000._wp    ! -  200km   - zgr_pse
      zxlim1 = 2010000._wp    !   2010km
      rpo (:,:,:) = rn_abp  ;   rpou(:,:,:) = rn_abp  ;   rpov(:,:,:) = rn_abp   ;   rpof(:,:,:) = rn_abp
      !
      DO jj = 2, jpjm1
         DO ji = 2, jpim1
           ! it is "largely" whithin the box
           IF ( gphit(ji,jj) > zylim0 .AND. gphit(ji,jj) < zylim1 .AND. &
              & glamt(ji,jj) > zxlim0 .AND. glamt(ji,jj) < zxlim1       )  THEN
              CALL zgr_pse (ji,jj,1,glamf,gphif,rpo , nT )
              CALL zgr_pse (ji,jj,1,glamt,gphit,rpof, nF )
              CALL zgr_pse (ji,jj,1,glamv,gphiv,rpou, nU )
              CALL zgr_pse (ji,jj,1,glamu,gphiu,rpov, nV )
           END IF
        END DO
      END DO
      !
      r1_rpo (:,:,:) = r1_abp ; r1_rpou(:,:,:) = r1_abp ; r1_rpov(:,:,:) = r1_abp
      bmpt   (:,:,:) = 0._wp  ; bmpu   (:,:,:) = 0._wp  ; bmpv   (:,:,:) = 0._wp
      !
      r1_rpof(:,:,:) = r1_abp
      !
      DO jj = 1, jpj
         DO ji = 1, jpi
           !
           ! thinner
           !bmpu(ji,jj, 1:jpkm1) = rn_fsp * ( 1._wp - rpou(ji,jj,1:jpkm1) )
           !bmpv(ji,jj, 1:jpkm1) = rn_fsp * ( 1._wp - rpov(ji,jj,1:jpkm1) )
           ! classic constant
           IF( rpou(ji,jj,1) < 1._wp )   bmpu(ji,jj, 1:jpkm1) = rn_fsp
           IF( rpov(ji,jj,1) < 1._wp )   bmpv(ji,jj, 1:jpkm1) = rn_fsp
           !
           r1_rpo (ji,jj,1:jpkm1) = 1._wp / rpo (ji,jj,1:jpkm1)
           r1_rpou(ji,jj,1:jpkm1) = 1._wp / rpou(ji,jj,1:jpkm1)
           r1_rpov(ji,jj,1:jpkm1) = 1._wp / rpov(ji,jj,1:jpkm1)
           r1_rpof(ji,jj,1:jpkm1) = 1._wp / rpof(ji,jj,1:jpkm1)
        END DO
      END DO
      !
      WHERE(rpo(:,:,1) <= rn_abp )
        rpo  (:,:,1) = rn_abp
      END WHERE
      WHERE(rpou(:,:,1) <= rn_abp )
        rpou  (:,:,1) = rn_abp
      END WHERE
      WHERE(rpov(:,:,1) <= rn_abp )
        rpov  (:,:,1) = rn_abp
      END WHERE
      WHERE(rpof(:,:,1) <= rn_abp )
        rpof  (:,:,1) = rn_abp
      END WHERE
      !
      ! definition du mask
      k_top(:,:) = 1    ! = ocean
      k_bot(:,:) = NINT( z2d(:,:) )
      WHERE(rpo(:,:,1) <= rn_abp )
        k_top(:,:) = 0    ! = land
        k_bot(:,:) = 0
      END WHERE
      !
      CALL lbc_lnk_multi( 'usrdef_zgr', rpo , 'T', 1._wp, r1_rpo  , 'T', 1._wp,                   &
             &                          rpou, 'U', 1._wp, r1_rpou , 'U', 1._wp, bmpu, 'U', 1._wp,    &
             &                          rpov, 'V', 1._wp, r1_rpov , 'V', 1._wp, bmpv, 'V', 1._wp,    &
             &                          rpof, 'F', 1._wp, r1_rpof , 'F', 1._wp, kfillmode=jpfillcopy )
# else
      ! Dans l'idéal, j'aimerais bien pouvoir me passer de cette boucle
      zylim0 =   10000._wp    !  +10km
      zylim1 = 2010000._wp    ! 2010km
      zxlim0 =   10000._wp    !  +10km
      zxlim1 = 2010000._wp    ! 2010km
      !
      DO jj = 1, jpj
         DO ji = 1, jpi
         ! if T point in the 2000 km x 2000 km domain
         ! IF ( gphit(ji,jj) > zylim0 .AND. gphit(ji,jj) < zylim1 .AND. &
         !   & glamt(ji,jj) > zxlim0 .AND. glamt(ji,jj) < zxlim1       )  THEN
         ! if U,V points are in the 2000 km x 2000 km domain
         IF ( gphiv(ji,jj) > zylim0 .AND. gphiv(ji,jj) < zylim1 .AND. &
            & glamu(ji,jj) > zxlim0 .AND. glamu(ji,jj) < zxlim1       )  THEN
         k_top(ji,jj) = 1    ! = ocean
         k_bot(ji,jj) = NINT( z2d(ji,jj) )
         ELSE
         k_top(ji,jj) = 0    ! = land
         k_bot(ji,jj) = 0
         END IF
         END DO
      END DO
      ! mask the lonely corners
      DO jj = 2, jpim1
         DO ji = 2, jpim1
         zcoeff = k_top(ji+1,jj) + k_top(ji,jj+1)   &
            +     k_top(ji-1,jj) + k_top(ji,jj-1)
         IF ( zcoeff <= 1._wp )   THEN
            k_top(ji,jj) = 0    ! = land
            k_bot(ji,jj) = 0
         END IF
         END DO
      END DO

# endif

   END SUBROUTINE zgr_msk_top_bot


   SUBROUTINE zgr_zco( pdept_1d, pdepw_1d, pe3t_1d, pe3w_1d,   &   ! in : 1D reference vertical coordinate
      &                pdept   , pdepw   ,                     &   ! out: 3D t & w-points depth
      &                pe3t    , pe3u    , pe3v   , pe3f   ,   &   !      vertical scale factors
      &                pe3w    , pe3uw   , pe3vw             )     !          -      -      -
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE zgr_zco  ***
      !!
      !! ** Purpose :   define the reference z-coordinate system
      !!
      !! ** Method  :   set 3D coord. arrays to reference 1D array
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:)    , INTENT(in   ) ::   pdept_1d, pdepw_1d          ! 1D grid-point depth       [m]
      REAL(wp), DIMENSION(:)    , INTENT(in   ) ::   pe3t_1d , pe3w_1d           ! 1D vertical scale factors [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(  out) ::   pdept, pdepw                ! grid-point depth          [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(  out) ::   pe3t , pe3u , pe3v , pe3f   ! vertical scale factors    [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(  out) ::   pe3w , pe3uw, pe3vw         !    -       -      -
      !
      INTEGER  ::   jk
      !!----------------------------------------------------------------------
      !
      DO jk = 1, jpk
         pdept(:,:,jk) = pdept_1d(jk)
         pdepw(:,:,jk) = pdepw_1d(jk)
         pe3t (:,:,jk) = pe3t_1d (jk)
         pe3u (:,:,jk) = pe3t_1d (jk)
         pe3v (:,:,jk) = pe3t_1d (jk)
         pe3f (:,:,jk) = pe3t_1d (jk)
         pe3w (:,:,jk) = pe3w_1d (jk)
         pe3uw(:,:,jk) = pe3w_1d (jk)
         pe3vw(:,:,jk) = pe3w_1d (jk)
      END DO
      !
   END SUBROUTINE zgr_zco


   SUBROUTINE zgr_pse( ki, kj, kk,                  &
      &                pglam,pgphi,prpo,            &
      &                cpoint                       )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE zgr_pse  ***
      !!
      !! ** Purpose :   Estimate the porosity field within the cell
      !!
      !! ** Method  :   find the intersection points with the boundaries
      !!                calculate the area land/water
      !!----------------------------------------------------------------------
      INTEGER                   , INTENT(in   ) ::   ki, kj, kk    ! coordinate of the center of the cell
      REAL, DIMENSION(:,:)      , INTENT(in   ) ::   pglam, pgphi  ! grid-point position          [m]
      REAL, DIMENSION(:,:,:)    , INTENT(inout) ::   prpo          ! porosity field
      INTEGER                   , INTENT(in   ) ::   cpoint        ! type of point in the C-grid
                                                                   ! T : 1, U : 2, V : 3, F : 4
      INTEGER  ::  jtype, jlen                     ! geometrical figure ; nearest land(=0)/water(=1) point
      REAL ::   z1d, z1_dx, zl                     ! dummy variable
      REAL, DIMENSION(2) ::   zA, zB, zC, zD, zM   ! coordinate array M(x,y)
      !
      INTEGER  ::   jkM                      ! local index of intersection points
      REAL,    DIMENSION(2)    :: zlm        ! save the lenght alond the edges
      INTEGER, DIMENSION(4)    :: zsp, zss   ! Sign Points, sum sign
                                             ! define the positions of points to the limit
      !!----------------------------------------------------------------------
      !
      z1_dx =   REAL(nn_AM98, wp) / rn_dx       ! [1/m] inverse gridspacing used
      !REAL :: dx = 1.
      !REAL :: z1_dx =   1.       ! [1/m] inverse gridspacing used
      !
      ! WRITE(*,*) 'BEGINNING of the routine zgr_pse'
      !                                      !==  Preparatory work  ==!
      SELECT CASE ( cpoint )                     !* Defining vertices
        CASE ( nT )                                               ! F coordinates
          zA(1) = pglam(ki-1,kj-1 ) ; zA(2) = pgphi(ki-1,kj-1 )
          zB(1) = pglam(ki  ,kj-1 ) ; zB(2) = pgphi(ki  ,kj-1 )
          zC(1) = pglam(ki  ,kj   ) ; zC(2) = pgphi(ki  ,kj   )
          zD(1) = pglam(ki-1,kj   ) ; zD(2) = pgphi(ki-1,kj   )
        CASE ( nF )                                               ! T coordinates
          zA(1) = pglam(ki  ,kj   ) ; zA(2) = pgphi(ki  ,kj   )
          zB(1) = pglam(ki+1,kj   ) ; zB(2) = pgphi(ki+1,kj   )
          zC(1) = pglam(ki+1,kj+1 ) ; zC(2) = pgphi(ki+1,kj+1 )
          zD(1) = pglam(ki  ,kj+1 ) ; zD(2) = pgphi(ki  ,kj+1 )
        CASE ( nU )                                               ! V coordinates
          zA(1) = pglam(ki  ,kj-1 ) ; zA(2) = pgphi(ki  ,kj-1 )
          zB(1) = pglam(ki+1,kj-1 ) ; zB(2) = pgphi(ki+1,kj-1 )
          zC(1) = pglam(ki+1,kj   ) ; zC(2) = pgphi(ki+1,kj   )
          zD(1) = pglam(ki  ,kj   ) ; zD(2) = pgphi(ki  ,kj   )
        CASE ( nV )                                               ! U coordinates
          zA(1) = pglam(ki-1,kj   ) ; zA(2) = pgphi(ki-1,kj   )
          zB(1) = pglam(ki  ,kj   ) ; zB(2) = pgphi(ki  ,kj   )
          zC(1) = pglam(ki  ,kj+1 ) ; zC(2) = pgphi(ki  ,kj+1 )
          zD(1) = pglam(ki-1,kj+1 ) ; zD(2) = pgphi(ki-1,kj+1 )
      END SELECT
      !
        ! WRITE(*,*) ' zA = ',zA
        ! WRITE(*,*) ' zB = ',zB
        ! WRITE(*,*) ' zC = ',zC
        ! WRITE(*,*) ' zD = ',zD
      !
      !                                       zss(3)
      !                                zsp(4)   |     zsp(3)
      !      D --------- C                 D --------- C
      !      |           |                 |           |
      !      |     +     |         zss(4)--|     +     |-- zss(2)
      !      |  (ki,kj)  |                 |           |
      !      A --------- B                 A --------- B
      !                                zsp(1)   |     zsp(2)
      !                                       zss(1)
      !
      ! !! case 1 : (x=)glam = 0             !* Defining position to the boundary
      ! !! 1 = is in water ; 0 in land
      ! zsp(:) = 0   ;   zss(:) = 0
      ! IF ( zA(1) > ( 0. )  )   zsp(1) = 1    ! A
      ! IF ( zB(1) > ( 0. )  )   zsp(2) = 1    ! B
      ! IF ( zC(1) > ( 0. )  )   zsp(3) = 1    ! C
      ! IF ( zD(1) > ( 0. )  )   zsp(4) = 1    ! D
      ! !
      !! case 1 : (x=)glam = 0             !* Defining position to the boundary
      !! 1 = is in water ; 0 in land
      zsp(:) = 0   ;   zss(:) = 0
      IF ( zA(1) > ( 0. )  )   zsp(1) = 1    ! A
      IF ( zB(1) > ( 0. )  )   zsp(2) = 1    ! B
      IF ( zC(1) > ( 0. )  )   zsp(3) = 1    ! C
      IF ( zD(1) > ( 0. )  )   zsp(4) = 1    ! D
      !
      ! !! case 2 : (x=)gphi = 2000 km       !* Defining position to the boundary
      ! !! 1 = is in water ; 0 in land
      ! zsp(:) = 0   ;   zss(:) = 0
      ! IF ( zA(2) < ( 200000. )  )   zsp(1) = 1    ! A
      ! IF ( zB(2) < ( 200000. )  )   zsp(2) = 1    ! B
      ! IF ( zC(2) < ( 200000. )  )   zsp(3) = 1    ! C
      ! IF ( zD(2) < ( 200000. )  )   zsp(4) = 1    ! D
      !
      zss(1) = zsp(1) + zsp(2)
      zss(2) = zsp(2) + zsp(3)
      zss(3) = zsp(3) + zsp(4)
      zss(4) = zsp(4) + zsp(1)
      !
      IF      ( SUM( zsp(:) ) == 3 )  THEN
         jtype = 1   ;   jlen = 0
         ! WRITE(*,*) 'land triangle'
      ELSE IF ( SUM( zsp(:) ) == 2 )  THEN
         jtype = 2   ;  jlen = 1
         ! WRITE(*,*) 'water trapeze'
      ELSE IF ( SUM( zsp(:) ) == 1 )  THEN
         jtype = 3    ;  jlen = 1
         ! WRITE(*,*) 'water triangle'
      ELSE IF ( pglam(ki,kj) > 0._wp ) THEN
         ! SUM( zsp(:) ) == 4
         ! prpo(ki,kj,kk) = prpo(ki,kj,kk) + 1._wp
         prpo(ki,kj,kk) =  1._wp
         ! full water
         RETURN
      ELSE
        ! full land
        RETURN
      ENDIF
      !
      ! WRITE(*,*) ' zsp = ',zsp(4), zsp(3)
      ! WRITE(*,*) '       ',zsp(1), zsp(2)
      !
      zlm(:) = 0.
      !                            !==  Searching for the intersection points  ==!
      !!                                !* NS west boundary
      zM(1) = 0.
      ! WRITE(*,*) ' zlm = ',zlm
      jkM = 1
       !    A ------- B
      IF ( zss(1) == 1 ) THEN
        zM(2) = weightedMx(zA,zB, z1_dx, 0.)
        !
        IF (zsp(1) == jlen) THEN
          ! WRITE(*,*) ' AM '
          zlm(jkM) = lenght(zA,zM)
        ELSE
          ! WRITE(*,*) ' MB '
          zlm(jkM) = lenght(zB,zM)
        ENDIF
        jkM = jkM + 1
        ! WRITE(*,*) ' '
        ! WRITE(*,*) ' zM = ', zM
        ! WRITE(*,*) ' zlm = ',zlm
      ENDIF
       !   C
       !   |
       !   |
       !   |
       !   B
      IF ( zss(2) == 1 ) THEN
        zM(2) = weightedMx(zB,zC, z1_dx,0.)
        !
        IF (zsp(2) == jlen) THEN
          zlm(jkM) = lenght(zB,zM)
        ELSE
          zlm(jkM) = lenght(zC,zM)
        ENDIF
        jkM = jkM + 1
        ! WRITE(*,*) ' '
        ! WRITE(*,*) ' BC '
        ! WRITE(*,*) ' zM = ', zM
        ! WRITE(*,*) ' zlm = ',zlm
      ENDIF
      !  !      D ------- C
      IF ( zss(3) == 1 ) THEN
        zM(2) = weightedMx(zC,zD, z1_dx,0.)
        !
        IF (zsp(3) == jlen) THEN
          ! WRITE(*,*) ' CM '
          zlm(jkM) = lenght(zC,zM)
        ELSE
          ! WRITE(*,*) ' MD '
          zlm(jkM) = lenght(zD,zM)
        ENDIF
        jkM = jkM + 1
        ! WRITE(*,*) ' '
        ! WRITE(*,*) ' zM = ', zM
        ! WRITE(*,*) ' zlm = ',zlm
      ENDIF
      !  !   D
      !      |
      !      |
      !      |
      !      A
      IF ( zss(4) == 1 ) THEN
        zM(2) = weightedMx(zD,zA, z1_dx, 0.)
        !
        IF (zsp(4) == jlen) THEN
          zlm(jkM) = lenght(zD,zM)
        ELSE
          zlm(jkM) = lenght(zA,zM)
        ENDIF
        jkM = jkM + 1
        ! WRITE(*,*) ' '
        ! WRITE(*,*) ' DA '
        ! WRITE(*,*) ' zM = ', zM
        ! WRITE(*,*) ' zlm = ',zlm
      ENDIF
      ! WRITE(*,*) 'ENDIF  zlm = ',zlm
      !
      !
      !                           !==  Calculating land area  ==!
      !  ratio of water upon the surface
      IF     (jtype == 1) THEN    ! land triangle
        z1d = 1. - 0.5 * zlm(1) * zlm(2) * z1_dx * z1_dx
      ELSEIF (jtype == 2) THEN    ! water trapeze
        z1d =      0.5 * ( zlm(1) + zlm(2) ) * z1_dx
      ELSEIF (jtype == 3) THEN    ! water triangle
        z1d =      0.5 * zlm(1) * zlm(2) * z1_dx * z1_dx
      ENDIF
      ! update the porosity field in += manner
      ! prpo(ki,kj,kk) = prpo(ki,kj,kk) + z1d
      prpo(ki,kj,kk) = z1d
      ! WRITE(*,*) 'prpo(ki,kj,kk)=',prpo(ki,kj,kk)
      ! ...
      ! NS east boundary
      ! WE south boundary
      ! WE north boundary
      !
      !
      ! WRITE(*,*) 'END of the routine zgr_pse'
      !
   END SUBROUTINE zgr_pse


   FUNCTION weightedMx(A,B,z1_dx,xl)  RESULT(f)
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE weightedM  ***
      !!
      !! ** Purpose :
      !!
      !! ** Method  :
      !!
      !!----------------------------------------------------------------------
      IMPLICIT NONE
      REAL, DIMENSION(2), INTENT(in) :: A,B
      REAL,                 INTENT(in) :: z1_dx, xl
      REAL                             ::  f                 ! result
      !!----------------------------------------------------------------------
      !
      f  =    (   A(2) * ( B(1) - xl     )    &
         &    +   B(2) * ( xl     - A(1) )    )   &
         &    /            ( B(1) - A(1) )
      !
   END FUNCTION weightedMx

   FUNCTION lenght(A,B)  RESULT(f)
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE weighted_lenght_X  ***
      !!
      !! ** Purpose :
      !!
      !! ** Method  :
      !!
      !!----------------------------------------------------------------------
      IMPLICIT NONE
      REAL, DIMENSION(2), INTENT(in) :: A,B
      REAL             ::  f                   ! result
      !!----------------------------------------------------------------------
      !
      f  = SQRT(   ( A(1) - B(1) ) * ( A(1) - B(1) )    &
         &     +   ( A(2) - B(2) ) * ( A(2) - B(2) )    )
      !
   END FUNCTION lenght


   !!======================================================================
END MODULE usrdef_zgr
